'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babylon = require('babylon');

var _babelTemplate = require('babel-template');

var _babelTemplate2 = _interopRequireDefault(_babelTemplate);

var _babelGenerator = require('babel-generator');

var _babelGenerator2 = _interopRequireDefault(_babelGenerator);

var _prettier = require('prettier');

var _prettier2 = _interopRequireDefault(_prettier);

var _babelTypes = require('babel-types');

var babelTypes = _interopRequireWildcard(_babelTypes);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const plugins = ['asyncGenerators', 'classProperties', 'decorators', 'doExpressions', 'exportExtensions', 'flow', 'functionSent', 'functionBind', 'jsx', 'objectRestSpread', 'dynamicImport', 'numericSeparator', 'optionalChaining', 'optionalCatchBinding'];

class AstHelper {
  constructor(logger) {
    this.logger = logger;
  }

  replaceNode(node, fileContent, template, args) {
    const tpl = (0, _babelTemplate2.default)(template);
    const ast = tpl(args);
    const replacement = (0, _babelGenerator2.default)(ast, { retainLines: true, compact: false }, fileContent.slice(node.start, node.end));

    let formatted = _prettier2.default.format(replacement.code, {
      semi: false,
      singleQuote: true
    });

    if (ast.type === 'ExpressionStatement' && formatted.charAt(0) === ';') {
      formatted = formatted.slice(1);
    }

    return formatted.trim();
  }

  parseCode(code) {
    const ast = (0, _babylon.parse)(code, { sourceType: 'module', plugins });
    return ast;
  }

  findNodes(code, location) {
    const ast = this.parseCode(code);
    let nodes = [];
    let ancestors = [];

    const walk = node => {
      if (!node) return;

      if (!this.isUnderCursor(node, location)) {
        return;
      }

      ancestors.push(node);

      const subNodesKeys = babelTypes.VISITOR_KEYS[node.type] || [];

      for (const key of subNodesKeys) {
        const subNodes = wrapInArray(node[key]);

        for (const n of subNodes) {
          walk(n);
        }
      }
    };

    walk(ast);

    return ancestors.reverse();
  }

  isUnderCursor(node, location) {
    const { start, end } = node.loc;
    const position = location.range.start;
    const line = position.line + 1;

    if (start.line > line) {
      return false;
    }

    if (start.line === line && start.column > position.character) {
      return false;
    }

    if (end.line < line) {
      return false;
    }

    if (end.line === line && end.column < position.character) {
      return false;
    }

    return true;
  }
}

exports.default = AstHelper;
function wrapInArray(el) {
  if (Array.isArray(el)) {
    return el;
  }
  return [el];
}