#!/usr/bin/env node
'use strict';

var _yargs = require('yargs');

var _yargs2 = _interopRequireDefault(_yargs);

var _connectionFromOptions = require('../utils/connectionFromOptions');

var _connectionFromOptions2 = _interopRequireDefault(_connectionFromOptions);

var _initializeLogging = require('../initializeLogging');

var _initializeLogging2 = _interopRequireDefault(_initializeLogging);

var _server = require('../server');

var _server2 = _interopRequireDefault(_server);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

//  weak

const cli = _yargs2.default.usage('JavaScript refactoring Language Service Command-Line Interface.\nUsage: $0 [args]').help('h').alias('h', 'help').option('node-ipc', {
  describe: 'Use node-ipc to communicate with the server. Useful for calling from a node.js client',
  type: 'string'
}).option('stdio', {
  describe: 'Use stdio to communicate with the server',
  type: 'string'
}).option('pipe', {
  describe: 'Use a pipe (with a name like --pipe=/tmp/named-pipe) to communicate with the server',
  type: 'string'
}).option('socket', {
  describe: 'Use a socket (with a port number like --socket=5051) to communicate with the server',
  type: 'number'
});

const argv = cli.argv;
const methods = ['node-ipc', 'stdio', 'pipe', 'socket'];

cliInvariant(methods.filter(m => argv[m] != null).length === 1, 'jsref requires exactly one valid connection option (node-ipc, stdio, pipe, or socket).');
const method = methods.find(m => argv[m] != null) || 'unknown';

let port;
if (method === 'socket') {
  cliInvariant(argv.socket, '--socket option requires port.');
  port = argv.socket;
}

const connection = (0, _connectionFromOptions2.default)(method, port);
(0, _initializeLogging2.default)(connection);

const server = new _server2.default(connection);

// Exit the process when stream closes from remote end.
process.stdin.on('close', () => {
  process.exit(0);
});

function cliInvariant(condition, ...msgs) {
  if (!condition) {
    /* eslint-disable no-console */
    console.error('ERROR:', ...msgs);
    console.error();
    /* eslint-enable */
    cli.showHelp();
    process.exit(1);
  }
}