'use strict';

var _log4js = require('log4js');

var _documentStorage = require('../../document-storage');

var _documentStorage2 = _interopRequireDefault(_documentStorage);

var _astHelper = require('../../ast-helper');

var _astHelper2 = _interopRequireDefault(_astHelper);

var _explicitReturnToImplicit = require('../explicit-return-to-implicit');

var _explicitReturnToImplicit2 = _interopRequireDefault(_explicitReturnToImplicit);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const mock = (0, _utils.createMock)(_explicitReturnToImplicit2.default);

test('should suggest fix', () => {
  const code = `const foo = () => { return 1; }`;
  const commands = mock.suggest(code, 0, 15);

  expect(commands.length).toBe(1);
  expect(commands[0].command).toBe('explicit_return_to_implicit');
  expect(commands[0].title).toMatch('Use implicit return');
});

test('should not suggest fix if arrow already has implicit return', () => {
  const code = `const foo = () => 1`;
  const commands = mock.suggest(code, 0, 15);

  expect(commands.length).toBe(0);
});

test('should not suggest fix if arrow has >1 statements', () => {
  const code = `const foo = () => {
  console.log('something')
  return 'nothing'
}`;
  const commands = mock.suggest(code, 0, 15);

  expect(commands.length).toBe(0);
});

test('should not suggest fix if arrow has no return statement', () => {
  const code = `const foo = () => {
  console.log('something')
}`;
  const commands = mock.suggest(code, 0, 15);

  expect(commands.length).toBe(0);
});

test('should convert body', () => {
  const code = `const foo = () => {return 1;}`;
  const edit = mock.edit(0, code, 0, 15);

  const expected = `() => 1`;

  const change = Object.values(edit.changes)[0];
  expect(change).toEqual([{
    newText: expected,
    range: { start: { line: 0, character: 12 }, end: { line: 0, character: 29 } }
  }]);
});

test('should copy params convert body', () => {
  const code = `const foo = (a) => {return a;}`;
  const edit = mock.edit(0, code, 0, 15);

  const expected = `a => a`;

  const change = Object.values(edit.changes)[0];
  expect(change).toEqual([{
    newText: expected,
    range: { start: { line: 0, character: 12 }, end: { line: 0, character: 30 } }
  }]);
});