'use strict';

var _log4js = require('log4js');

var _documentStorage = require('../../document-storage');

var _documentStorage2 = _interopRequireDefault(_documentStorage);

var _astHelper = require('../../ast-helper');

var _astHelper2 = _interopRequireDefault(_astHelper);

var _functionToArrow = require('../function-to-arrow');

var _functionToArrow2 = _interopRequireDefault(_functionToArrow);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const mock = (0, _utils.createMock)(_functionToArrow2.default);

test('should suggest fix', () => {
  const code = `function hello() {
}`;
  const commands = mock.suggest(code, 0, 3);

  expect(commands.length).toBe(1);
  expect(commands[0].command).toBe('function_to_arrow');
  expect(commands[0].title).toMatch('hello');
});

test('should suggest fix on internal function', () => {
  const code = `function hello() {
  function cool() {
  }
}`;
  const commands = mock.suggest(code, 1, 4);

  expect(commands.length).toBe(1);
  expect(commands[0].command).toBe('function_to_arrow');
  expect(commands[0].title).toMatch('cool');
});

test('should create simple edit', () => {
  const code = `function hello() {}`;
  const edit = mock.edit(0, code, 0, 3);

  expect(edit).toEqual({
    changes: {
      'file://testfile.js': [{
        newText: 'const hello = () => {}',
        range: { end: { character: 19, line: 0 }, start: { character: 0, line: 0 } }
      }]
    }
  });
});

test('should copy params', () => {
  const code = `function hello({params, title = 1}, ...rest) {}`;
  const edit = mock.edit(0, code, 0, 3);

  expect(edit.changes['file://testfile.js'][0].newText).toBe('const hello = ({ params, title = 1 }, ...rest) => {}');
});