'use strict';

var _log4js = require('log4js');

var _documentStorage = require('../../document-storage');

var _documentStorage2 = _interopRequireDefault(_documentStorage);

var _astHelper = require('../../ast-helper');

var _astHelper2 = _interopRequireDefault(_astHelper);

var _implicitReturnToExplicit = require('../implicit-return-to-explicit');

var _implicitReturnToExplicit2 = _interopRequireDefault(_implicitReturnToExplicit);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const mock = (0, _utils.createMock)(_implicitReturnToExplicit2.default);

test('should suggest fix', () => {
  const code = `const foo = () => 1`;
  const commands = mock.suggest(code, 0, 15);

  expect(commands.length).toBe(1);
  expect(commands[0].command).toBe('implicit_return_to_explicit');
  expect(commands[0].title).toMatch('Use explicit return');
});

test('should not suggest fix if arrow has block', () => {
  const code = `const foo = () => { return 1 }`;
  const commands = mock.suggest(code, 0, 15);

  expect(commands.length).toBe(0);
});

test('should create body with return', () => {
  const code = `const foo = () => 1;`;
  const edit = mock.edit(0, code, 0, 15);

  const expected = `() => {
  return 1
}`;

  const change = Object.values(edit.changes)[0];
  expect(change).toEqual([{
    newText: expected,
    range: { start: { line: 0, character: 12 }, end: { line: 0, character: 19 } }
  }]);
});

test('should work inside expression', () => {
  const code = `console.log(() => 1);`;
  const edit = mock.edit(0, code, 0, 15);

  const expected = `() => {
  return 1
}`;

  const change = Object.values(edit.changes)[0];
  expect(change).toEqual([{
    newText: expected,
    range: { start: { line: 0, character: 12 }, end: { line: 0, character: 19 } }
  }]);
});

test('should copy params', () => {
  const code = `console.log(({b}) => b);`;
  const edit = mock.edit(0, code, 0, 15);

  const expected = `({ b }) => {
  return b
}`;

  const change = Object.values(edit.changes)[0];
  expect(change).toEqual([{
    newText: expected,
    range: { start: { line: 0, character: 12 }, end: { line: 0, character: 22 } }
  }]);
});