'use strict';

var _log4js = require('log4js');

var _documentStorage = require('../../document-storage');

var _documentStorage2 = _interopRequireDefault(_documentStorage);

var _astHelper = require('../../ast-helper');

var _astHelper2 = _interopRequireDefault(_astHelper);

var _reactFunctionToClass = require('../react-function-to-class');

var _reactFunctionToClass2 = _interopRequireDefault(_reactFunctionToClass);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const mock = (0, _utils.createMock)(_reactFunctionToClass2.default);

test('should suggest fix', () => {
  const code = `import React from 'react';

function Page(props) {
  return <div>{props.title}</div>
}
`;
  const commands = mock.suggest(code, 3, 4);

  expect(commands.length).toBe(1);
  expect(commands[0].command).toBe('react-function-to-class');
  expect(commands[0].title).toMatch('Convert "Page" to React class');
});

test('should create simple class', () => {
  const code = `import React from 'react';

function Page(props) {
  return <div>{props.title}</div>
}
`;
  const edit = mock.edit(0, code, 3, 4);

  const expected = `class Page extends React.Component {
  render() {
    const props = this.props
    return <div>{props.title}</div>
  }
}`;

  const change = Object.values(edit.changes)[0];
  expect(change).toEqual([{
    newText: expected,
    range: { start: { line: 2, character: 0 }, end: { line: 4, character: 1 } }
  }]);
});

test('should convert destructuring in argument', () => {
  const code = `import React from 'react';

function Page({title}) {
  return <div>{title}</div>
}
`;
  const edit = mock.edit(0, code, 3, 4);

  const expected = `class Page extends React.Component {
  render() {
    const { title } = this.props
    return <div>{title}</div>
  }
}`;

  const change = Object.values(edit.changes)[0];
  expect(change).toEqual([{
    newText: expected,
    range: { start: { line: 2, character: 0 }, end: { line: 4, character: 1 } }
  }]);
});