'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _log4js = require('log4js');

var _utils = require('./utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

const DEFAULT_NAME = 'replace_me_name';

class ArrowToFunction {

  constructor(opts) {
    this.type = 'arrow_to_function';
    this.ds = opts.documentStorage;
    this.ast = opts.astHelper;
    this.logger = (0, _log4js.getLogger)(this.type);
  }

  suggestCommands(location) {
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'ArrowFunctionExpression');
    if (node) {
      return [{
        title: 'Convert arrow function to regular',
        command: this.type,
        arguments: [location]
      }];
    }

    return [];
  }

  createEdit(args) {
    // TODO check args
    const location = args[0];
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'ArrowFunctionExpression');

    if (node) {
      const index = nodes.indexOf(node);
      const parent = nodes[index + 1];

      const isDeclaration = parent.type === 'VariableDeclarator' && parent.init === node;
      const name = isDeclaration ? parent.id : t.identifier(DEFAULT_NAME);

      const args = { PARAMS: node.params, NAME: name };

      let template = '(function NAME(PARAMS) {';

      if (node.body.type === 'BlockStatement') {
        template += ' BODY ';
        args.BODY = node.body.body;
      } else {
        template += 'return BODY';
        args.BODY = node.body;
      }

      template += '})';

      const nodeToReplace = isDeclaration ? nodes[index + 2] : node;
      const newCode = this.ast.replaceNode(nodeToReplace, code, template, args);

      return (0, _utils.createReplacementEdit)(location.uri, nodeToReplace, newCode);
    }
  }
}
exports.default = ArrowToFunction;