'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _log4js = require('log4js');

var _utils = require('./utils');

class ExplicitReturnToImplicit {

  constructor(opts) {
    this.type = 'explicit_return_to_implicit';
    this.ds = opts.documentStorage;
    this.ast = opts.astHelper;
    this.logger = (0, _log4js.getLogger)(this.type);
  }

  suggestCommands(location) {
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'ArrowFunctionExpression');
    if (node && node.body.type === 'BlockStatement' && node.body.body.length === 1 && node.body.body[0].type === 'ReturnStatement') {
      return [{
        title: 'Use implicit return',
        command: this.type,
        arguments: [location]
      }];
    }

    return [];
  }

  createEdit(args) {
    // TODO check args
    const location = args[0];
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'ArrowFunctionExpression');

    if (node) {
      const template = `(PARAMS) => EXP`;
      const args = {
        EXP: node.body.body[0].argument,
        PARAMS: node.params
      };

      const newCode = this.ast.replaceNode(node, code, template, args);

      return (0, _utils.createReplacementEdit)(location.uri, node, newCode);
    }
  }
}
exports.default = ExplicitReturnToImplicit;