'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _log4js = require('log4js');

var _utils = require('./utils');

class FunctionToArrow {

  constructor(opts) {
    this.type = 'function_to_arrow';
    this.ds = opts.documentStorage;
    this.ast = opts.astHelper;
    this.logger = (0, _log4js.getLogger)(this.type);
  }

  suggestCommands(location) {
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'FunctionDeclaration');
    if (node) {
      return [{
        title: `Convert "${node.id.name}" to arrow function`,
        command: this.type,
        arguments: [location]
      }];
    }

    return [];
  }

  createEdit(args) {
    // TODO check args
    const location = args[0];
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'FunctionDeclaration');

    if (node) {
      const arrowTemplate = 'const NAME = (PARAMS) => BODY';
      const newCode = this.ast.replaceNode(node, code, arrowTemplate, {
        NAME: node.id,
        BODY: node.body,
        PARAMS: node.params
      });

      return (0, _utils.createReplacementEdit)(location.uri, node, newCode);
    }
  }
}
exports.default = FunctionToArrow;