'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _log4js = require('log4js');

var _utils = require('./utils');

class ImplicitReturnToExplicit {

  constructor(opts) {
    this.type = 'implicit_return_to_explicit';
    this.ds = opts.documentStorage;
    this.ast = opts.astHelper;
    this.logger = (0, _log4js.getLogger)(this.type);
  }

  suggestCommands(location) {
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'ArrowFunctionExpression');
    if (node && node.body.type !== 'BlockStatement') {
      return [{
        title: 'Use explicit return',
        command: this.type,
        arguments: [location]
      }];
    }

    return [];
  }

  createEdit(args) {
    // TODO check args
    const location = args[0];
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'ArrowFunctionExpression');

    if (node) {
      const template = `(PARAMS) => { return BODY }`;
      const args = {
        BODY: node.body,
        PARAMS: node.params
      };

      const newCode = this.ast.replaceNode(node, code, template, args).replace(/;$/, '');

      return (0, _utils.createReplacementEdit)(location.uri, node, newCode);
    }
  }
}
exports.default = ImplicitReturnToExplicit;