'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _log4js = require('log4js');

var _utils = require('./utils');

function isReactImport(node) {
  return node.type === 'ImportDeclaration' && node.source && node.source.type === 'StringLiteral' && node.source.value === 'react';
}

function hasJsxReturn(node) {
  const returnStatement = node.body.body.find(i => i.type === 'ReturnStatement');
  if (!returnStatement) {
    return false;
  }

  if (returnStatement.argument && returnStatement.argument.type === 'JSXElement') {
    return true;
  }

  return false;
}

class ReactFunctionToClass {

  constructor(opts) {
    this.type = 'react-function-to-class';
    this.ds = opts.documentStorage;
    this.ast = opts.astHelper;
    this.logger = (0, _log4js.getLogger)(this.type);
  }

  suggestCommands(location) {
    const code = this.ds.get(location.uri);
    const ast = this.ast.parseCode(code);

    const isReactModule = ast.program.body.some(isReactImport);
    if (!isReactModule) {
      return [];
    }

    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'FunctionDeclaration');
    if (node && hasJsxReturn(node)) {
      return [{
        title: `Convert "${node.id.name}" to React class`,
        command: this.type,
        arguments: [location]
      }];
    }

    return [];
  }

  createEdit(args) {
    // TODO check args
    const location = args[0];
    const code = this.ds.get(location.uri);
    const nodes = this.ast.findNodes(code, location);
    const node = nodes.find(n => n.type === 'FunctionDeclaration');

    if (node) {
      const arrowTemplate = `class ${node.id.name} extends React.Component {
  render() {
    const PARAM = this.props
    BODY
  }
}`;
      const newCode = this.ast.replaceNode(node, code, arrowTemplate, {
        BODY: node.body.body,
        PARAM: node.params[0]
      }).replace(/\n+/g, '\n');

      return (0, _utils.createReplacementEdit)(location.uri, node, newCode);
    }
  }
}
exports.default = ReactFunctionToClass;