'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = connectionFromOptions;

var _vscodeJsonrpc = require('vscode-jsonrpc');

var _vscodeLanguageserver = require('vscode-languageserver');

var _log4js = require('log4js');

var _net = require('net');

var _net2 = _interopRequireDefault(_net);

var _stream = require('stream');

var _stream2 = _interopRequireDefault(_stream);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function connectionFromOptions(method, port) {
  let reader;
  let writer;
  let server;

  switch (method) {
    case 'socket':
      if (!port) {
        throw new Error('port required');
      }
      // For socket connection, the message connection needs to be
      // established before the server socket starts listening.
      // Do that, and return at the end of this block.
      writer = new _stream2.default.PassThrough();
      reader = new _stream2.default.PassThrough();
      server = _net2.default.createServer(socket => {
        server.close();
        socket.pipe(writer);
        reader.pipe(socket);
      }).listen(port);
      break;
    case 'stdio':
      reader = process.stdin;
      writer = process.stdout;
      break;
    case 'node-ipc':
    default:
      reader = new _vscodeJsonrpc.IPCMessageReader(process);
      writer = new _vscodeJsonrpc.IPCMessageWriter(process);
      break;
  }

  return (0, _vscodeLanguageserver.createConnection)(reader, writer);
}